# -*- coding: UTF-8 -*-

import nes_common.logger as logger
import sys
from artix.common.progress import Progress
from nes_common import nesconfig
from nes_common import sql_operation


class DataIntegrityError(Exception):
    pass


class Executor(object):
    def __init__(self, shopCode, connection, context):
        self.log = logger.getLogger('Shop', shopCode)
        self.connection = connection
        self.context = context
        self.trickyCounter = 0
        self.commandCommitCount = nesconfig.getInt('mysql', 'commandCommitCount', 5000)
        self.progress = Progress(self.log, nesconfig.getBool('MAIN', 'testProgressMode', False))

    def startTransaction(self):
        cur = self.connection.cursor()
        cur.execute('START TRANSACTION;')

    def execute(self, sql):
        try:
            sql.execute(self.context)
        except DataIntegrityError:
            self.log.error('data integrity error. exiting')
            sys.exit(1)

    def executeMany(self):
        sql_operation._Delete.execMany(self.context)
        sql_operation._Update.execMany(self.context)
        sql_operation._Insert.execMany(self.context)

    def commit(self):
        if self.trickyCounter >= self.commandCommitCount:
            self.executeMany()
            self.connection.commit()
            self.trickyCounter = 0
        self.trickyCounter += 1

    def rollback(self):
        self.connection.rollback()

    def flushTables(self):
        cur = self.connection.cursor()
        cur.execute('FLUSH tables;')

    def startUpload(self, messageId):
        self.log.info('Начало загрузки справочника')

        try:
            sql_operation._Insert.cache.clear()
            sql_operation._Update.cache.clear()
            sql_operation._Select.cache.clear()
            sql_operation._Select.cacheUnite.clear()
            sql_operation._Delete.cache.clear()
            sql_operation._Delete.cacheUnite.clear()
            sql_operation._Insert.cacheValues.clear()
            sql_operation._Update.cacheValues.clear()
            sql_operation._Delete.cacheValues.clear()
        except Exception as e:
            self.log.error('Error clear cache. %s' % e)

        self.flushTables()
        self.progress.initNesProgress(messageId)

    def finishUpload(self):
        self.log.info(u'Завершение загрузки справочника')
        self.executeMany()
        if not self.context.finalize():
            self.log.error('data integrity error. exiting')
            sys.exit(1)
        self.connection.commit()
        self.log.info(u"Сброс кэша запросов")
        self.flushTables()
        self.progress.removeProgresses()

    def updateProgress(self):
        """ Обновление прогресса загрузки """
        self.progress.updateNesProgress()
